﻿using System.Security.Authentication;
using System.Security.Claims;
using Hotline.Repository.SqlSugar;
using Hotline.Users;
using IdentityModel;
using SqlSugar;
using XF.Domain.Authentications;
using XF.Domain.Dependency;
using XF.Domain.Exceptions;
using XF.Utility.AppIdentityModel;

namespace Hotline.Api.Token
{
    public class DefaultSessionContext : ISessionContext, IScopeDependency
    {
        public DefaultSessionContext(IHttpContextAccessor httpContextAccessor, ISugarUnitOfWork<HotlineDbContext> uow)
        {
            var httpContext = httpContextAccessor.HttpContext;
            if (httpContext is null)
                //throw new ArgumentNullException(nameof(httpContext));
                return;

            var user = httpContext.User;
            //UserId = user.FindFirstValue(JwtClaimTypes.Id);
            UserId = user.FindFirstValue(ClaimTypes.NameIdentifier);
            UserName = user.FindFirstValue(AppClaimTypes.UserDisplayName);
            Phone = user.FindFirstValue(JwtClaimTypes.PhoneNumber);
            //Roles = user.Claims.Where(d => d.Type == JwtClaimTypes.Role).Select(d => d.Value).ToArray();
            Roles = user.Claims.Where(d => d.Type == ClaimTypes.Role).Select(d => d.Value).ToArray();
            OrgId = user.FindFirstValue(AppClaimTypes.DepartmentId);
            OrgCode = user.FindFirstValue(AppClaimTypes.DepartmentCode);
            OrgName = user.FindFirstValue(AppClaimTypes.DepartmentName);
        }

        /// <summary>
        /// Id of current tenant or null for host
        /// </summary>
        public string? UserId { get; }

        /// <summary>
        /// Id of current user or throw Exception for guest
        /// </summary>
        /// <exception cref="AuthenticationException"></exception>
        public string RequiredUserId => UserId ?? throw new ArgumentNullException();

        public string? UserName { get; }

        public string? Phone { get; }

        /// <summary>
        /// Roles
        /// </summary>
        public string[] Roles { get; }

        public string? OrgId { get; set; }
        public string? OrgCode { get; set; }
        public string? OrgName { get; set; }
        public string RequiredOrgId => OrgId ?? throw new ArgumentNullException();
        public string RequiredOrgCode => OrgCode ?? throw new ArgumentNullException();
    }
}
