﻿using Hotline.CallCenter.Tels;
using Hotline.FlowEngine.Notifications;
using Hotline.KnowledgeBase;
using Hotline.Orders;
using Hotline.Settings;
using MediatR;

namespace Hotline.Application.Handlers.FlowEngine;

public class EndWorkflowHandler : INotificationHandler<EndWorkflowNotify>
{

    private readonly IKnowledgeDomainService _knowledgeDomainService;
    private readonly IOrderDomainService _orderDomainService;
    private readonly ITelDomainService _telDomainService;

    public EndWorkflowHandler(
        IKnowledgeDomainService knowledgeDomainService,
        IOrderDomainService orderDomainService,
        ITelDomainService telDomainService
        )
    {
        _knowledgeDomainService = knowledgeDomainService;
        _orderDomainService = orderDomainService;
        _telDomainService = telDomainService;
    }

    /// <summary>Handles a notification</summary>
    /// <param name="notification">The notification</param>
    /// <param name="cancellationToken">Cancellation token</param>
    public async Task Handle(EndWorkflowNotify notification, CancellationToken cancellationToken)
    {
        var workflow = notification.Workflow;

        switch (workflow.ModuleCode)
        {
            case WorkflowModuleConsts.KnowledgeAdd://新增知识库
            //case WorkflowModuleConsts.KnowledgeUpdate://修改知识库
            case WorkflowModuleConsts.KnowledgeDelete://删除知识库
                await _knowledgeDomainService.EndWorkKnowledge(workflow, cancellationToken);
                break;

            case WorkflowModuleConsts.TelRestApply:
                await _telDomainService.TelRestApplyPassAsync(workflow.ExternalId, cancellationToken);
                break;

            case WorkflowModuleConsts.OrderManage:
                await _orderDomainService.ManageFlowEndAsync(workflow.ExternalId, cancellationToken);
                break;

        }
    }
}