﻿using Hotline.FlowEngine.Notifications;
using Hotline.Orders;
using Hotline.Settings;
using MediatR;
using Microsoft.Extensions.Logging;

namespace Hotline.Application.Handlers.FlowEngine;

public class JumpHandler : INotificationHandler<JumpNotify>
{
    private readonly IOrderDomainService _orderDomainService;
    private readonly ILogger<JumpHandler> _logger;

    public JumpHandler(
        IOrderDomainService orderDomainService,
        ILogger<JumpHandler> logger)
    {
        _orderDomainService = orderDomainService;
        _logger = logger;
    }

    /// <summary>Handles a notification</summary>
    /// <param name="notification">The notification</param>
    /// <param name="cancellationToken">Cancellation token</param>
    public async Task Handle(JumpNotify notification, CancellationToken cancellationToken)
    {
        var workflow = notification.Workflow;
        var data = notification.Dto;

        switch (workflow.ModuleCode)
        {
            case WorkflowModuleConsts.OrderManage:
                await _orderDomainService.ManageFlowJumpAsync(workflow.Id, notification.FlowAssignMode, cancellationToken);
                break;
        }
    }
}