﻿using Hotline.FlowEngine.Notifications;
using Hotline.Orders;
using Hotline.Settings;
using MediatR;

namespace Hotline.Application.Handlers.FlowEngine;

public class OrderRecallFinalManageHandler : INotificationHandler<OrderRecallFinalManageNotify>
{
    private readonly IOrderDomainService _orderDomainService;

    public OrderRecallFinalManageHandler(IOrderDomainService orderDomainService)
    {
        _orderDomainService = orderDomainService;
    }

    /// <summary>Handles a notification</summary>
    /// <param name="notification">The notification</param>
    /// <param name="cancellationToken">Cancellation token</param>
    public async Task Handle(OrderRecallFinalManageNotify notification, CancellationToken cancellationToken)
    {
        var workflow = notification.Workflow;

        switch (workflow.ModuleCode)
        {
            case WorkflowModuleConsts.OrderManage:
                await _orderDomainService.RecallFinalManageAsync(notification.Workflow.ExternalId, cancellationToken);
                break;
        }

    }
}