﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Hotline.Share.Dtos.FlowEngine;
using Hotline.Share.Dtos.Hotspots;
using Hotline.Share.Enums.Order;
using XF.Utility.EnumExtensions;

namespace Hotline.Share.Dtos.Order
{
    public class OrderDto : UpdateOrderDto
    {
        #region 工单属性

        /// <summary>
        /// 当前节点名称
        /// </summary>
        public string CurrentStepName { get; set; }

        /// <summary>
        /// 到达当前节点时间
        /// </summary>
        public DateTime? CurrentStepTime { get; set; }

        /// <summary>
        /// 工单状态
        /// </summary>
        public EOrderStatus Status { get; set; }

        /// <summary>
        /// 开始时间
        /// </summary>
        public DateTime? StartTime { get; set; }

        /// <summary>
        /// 过期时间
        /// </summary>
        public DateTime? ExpiredTime { get; set; }

        /// <summary>
        /// 工单办理完成时间，实际办理部门的办理时间（汇总节点的上一个非汇总节点办理完成时间，开启会签时清空该字段）
        /// </summary>
        public DateTime? CompleteTime { get; set; }

        /// <summary>
        /// 归档时间（暂为流程结束时间，因流程结束自动归档）
        /// </summary>
        public DateTime? FiledTime { get; set; }

        /// <summary>
        /// 过期状态
        /// </summary>
        public EExpiredStatus ExpiredStatus { get; set; }

        #endregion

        public string CreationTime { get; set; }

        public string StatusText => Status.GetDescription();

        public string? ContactMask { get; set; }

        /// <summary>
        /// 工单编码（202201010001）
        /// </summary>
        public string No { get; set; }

        /// <summary>
        /// 受理人姓名
        /// </summary>
        public string EmployeeName { get; set; }

        /// <summary>
        /// 受理人工号
        /// </summary>
        public string? EmployeeStaffNo { get; set; }

        /// <summary>
        /// 处理方式（直办、交办）
        /// </summary>
        public EProcessType ProcessType { get; set; }

        /// <summary>
        /// 进展情况
        /// </summary>
        public EProgress Progress { get; set; }

        public string WorkflowId { get; set; }
        public WorkflowDto Workflow { get; set; }

        /// <summary>
        /// 来源渠道
        /// </summary>
        public string ChannelText => Channel.GetDescription();

        /// <summary>
        /// 来电/信人性别
        /// </summary>
        public string FromGenderText => FromGender.GetDescription();

        /// <summary>
        /// 来电/信人身份
        /// </summary>
        public string IdentityTypeText => IdentityType.GetDescription();

        /// <summary>
        /// 工单类型
        /// </summary>
        public string OrderTypeText => OrderType.GetDescription();

        /// <summary>
        /// 受理类型
        /// </summary>
        public string AcceptTypeText => AcceptType.GetDescription();

        /// <summary>
        /// 紧急程度
        /// </summary>
        public string EmergencyLevelText => EmergencyLevel.GetDescription();

        /// <summary>
        /// 超期时间描述（需求：超期3天、0.5天后超期）
        /// </summary>
        public string ExpiredText => CalculateExpiredText();

        private string CalculateExpiredText()
        {
            //todo 完整处理方案：1.创建待过期表，Id,过期时间，即将过期时间等字段。2.延迟消息通知处理过期，删除子表数据，处理order过期状态字段。3.此处即可采用expiredStatus进行判断
            //todo 目前暂时采用过期时间计算
            if (!ExpiredTime.HasValue)
                return string.Empty;
            var days = (DateTime.Now - ExpiredTime.Value).TotalDays;
            switch (days)
            {
                case > 0:
                    return $"超期{days:N1}天";
                case < 0:
                    return $"{days:N1}天后超期";
                default:
                    return "超期不到1天";
            }
        }

        public HotspotDto? Hotspot { get; set; }

    }

    public class UpdateOrderDto : AddOrderDto
    {
        public string Id { get; set; }
    }

    public class AddOrderDto : PositionDto
    {
        #region 来电信息

        /// <summary>
        /// 来源渠道
        /// </summary>
        public EChannel Channel { get; set; }

        /// <summary>
        /// 渠道为电话时，此字段存在
        /// </summary>
        public string? CallId { get; set; }

        ///// <summary>
        ///// 接线员工(userId)
        ///// </summary>
        //public string EmployeeId { get; set; }

        /// <summary>
        /// 来电号码
        /// </summary>
        public string? FromPhone { get; set; }

        /// <summary>
        /// 转接号码（转接来源）
        /// </summary>
        public string? TransferPhone { get; set; }

        /// <summary>
        /// 来电/信人姓名
        /// </summary>
        public string FromName { get; set; }

        /// <summary>
        /// 来电/信人性别
        /// </summary>
        public EGender FromGender { get; set; }

        /// <summary>
        /// 来电/信人身份
        /// </summary>
        public EIdentityType IdentityType { get; set; }

        /// <summary>
        /// 证件类型
        /// </summary>
        public string? LicenceTypeCode { get; set; }

        public string? LicenceType { get; set; }

        /// <summary>
        /// 证件号码
        /// </summary>
        public string? LicenceNo { get; set; }

        /// <summary>
        /// 年龄段
        /// </summary>
        public string? AgeRangeCode { get; set; }

        public string? AgeRange { get; set; }

        /// <summary>
        /// 联系电话
        /// </summary>
        public string? Contact { get; set; }

        /// <summary>
        /// 是否接受短信，勾选校验手机号
        /// </summary>
        public bool AcceptSms { get; set; }

        /// <summary>
        /// 是否已发送短信
        /// </summary>
        public bool SmsSended { get; set; }

        public string SmsSendedText => SmsSended ? "已发送" : "未发送";

        /// <summary>
        /// 是否需要联系
        /// </summary>
        public bool NeedContact { get; set; }

        /// <summary>
        /// 工作单位(当“来电/信人身份”为“企业”时必填，其他情况非必填)
        /// </summary>
        public string? Company { get; set; }

        #endregion

        #region 诉求信息

        /// <summary>
        /// 工单类型
        /// </summary>
        public EOrderType OrderType { get; set; }

        /// <summary>
        /// 受理类型
        /// </summary>
        public EAcceptType AcceptType { get; set; }

        /// <summary>
        /// 紧急程度
        /// </summary>
        public EEmergencyLevel EmergencyLevel { get; set; } = EEmergencyLevel.Normal;

        public string Title { get; set; }

        /// <summary>
        /// 热点
        /// </summary>
        public string HotspotId { get; set; }
        public string HotspotName { get; set; }
        public string HotspotSpliceName { get; set; }

        /// <summary>
        /// 外部数据（为前端提供级联功能）
        /// </summary>
        public string HotspotExternal { get; set; }

        /// <summary>
        /// 事发时间
        /// </summary>
        public DateTime? IncidentTime { get; set; }

        /// <summary>
        /// 重复工单
        /// </summary>
        public string? DuplicateId { get; set; }

        public string? DuplicateTitle { get; set; }

        /// <summary>
        /// 推送分类
        /// </summary>
        public string? PushTypeCode { get; set; }

        public string? PushType { get; set; }

        /// <summary>
        /// 附件
        /// </summary>
        public List<string> Additions { get; set; } = new();

        /// <summary>
        /// 诉求内容
        /// </summary>
        public string Content { get; set; }

        #endregion

        /// <summary>
        /// 来源，区分省平台或110等其他平台同步过来的工单
        /// </summary>
        public ESource Source { get; set; }

        /// <summary>
        /// 投诉
        /// </summary>
        public OrderComplainDto OrderComplain { get; set; }

        /// <summary>
        /// 举报
        /// </summary>
        public OrderReportDto OrderReport { get; set; }
    }
}
