﻿using Hotline.Share.Dtos.FlowEngine;
using Hotline.Share.Enums.FlowEngine;
using SqlSugar;

namespace Hotline.FlowEngine.Definitions;

/// <summary>
/// 节点基础信息
/// </summary>
public class StepBasic
{
    public string Name { get; set; }

    /// <summary>
    /// 模板内唯一
    /// </summary>
    public string Code { get; set; }

    public EStepType StepType { get; init; }

    /// <summary>
    /// 办理者类型
    /// </summary>
    public EHandlerType HandlerType { get; set; }

    /// <summary>
    /// 只允许该角色下本部门人办理,否则该角色下所有人均可办理（办理者类型为角色时此字段有值）
    /// </summary>
    public bool? OnlySelfOrg { get; set; }

    /// <summary>
    /// 办理者分类（或是直接保存办理者）
    /// <example>
    /// 根据类型可能为：roles, orgLevels, orgTypes, orgCodes, userIds
    /// </example>
    /// </summary>
    public List<IdName> HandlerClassifies { get; set; } = new();

    /// <summary>
    /// 当前环节会签模式
    /// </summary>
    public ECountersignMode CountersignMode { get; set; }

    /// <summary>
    /// 发起会签节点code（不支持发起会签节点无此字段）
    /// </summary>
    public string? CountersignStartStepCode { get; set; }

    /// <summary>
    /// 会签汇总节点code（不支持发起会签节点无此字段）
    /// </summary>
    public string? CountersignEndStepCode { get; set; }

}