﻿using Hotline.FlowEngine.Workflows;
using Hotline.Share.Dtos.FlowEngine;
using XF.Domain.Entities;

namespace Hotline.FlowEngine;

public class FlowAssignMode
{
    //public FlowAssignMode(EFlowAssignType flowAssignType, List<HandlerGroupItem> handlerObjects)
    //{
    //    FlowAssignType = flowAssignType;
    //    HandlerObjects = handlerObjects;
    //}

    /// <summary>
    /// 流程指派类型
    /// </summary>
    public EFlowAssignType FlowAssignType { get; set; }

    /// <summary>
    /// 办理对象（UserIds/OrgCodes）
    /// </summary>
    public List<HandlerGroupItem> HandlerObjects { get; set; }

    /// <summary>
    /// 创建mode对象实例，自动生成一个groupId
    /// </summary>
    /// <param name="flowAssignType"></param>
    /// <param name="handlers"></param>
    /// <returns></returns>
    public static FlowAssignMode Create(EFlowAssignType flowAssignType, List<IdName> handlers, bool multiGroups = false)
    {
        if (handlers == null || !handlers.Any())
            return new();
        List<HandlerGroupItem> handlerObjects;
        if (multiGroups)
        {
            handlerObjects = handlers.Select(d => new HandlerGroupItem
            {
                Id = d.Id,
                Name = d.Name,
                GroupId = Guid.NewGuid().ToString(),
            }).ToList();
        }
        else
        {
            var groupId = Guid.NewGuid().ToString();
            handlerObjects = handlers.Select(d => new HandlerGroupItem
            {
                Id = d.Id,
                Name = d.Name,
                GroupId = groupId
            }).ToList();
        }

        return new FlowAssignMode
        {
            FlowAssignType = flowAssignType,
            HandlerObjects = handlerObjects,
        };
    }

    /// <summary>
    /// 办理人/办理部门（UserIds/OrgCodes）
    /// </summary>
    /// <returns></returns>
    public List<string> GetHandlers() =>
        HandlerObjects.Select(d => d.Id).ToList();
}