﻿using Hotline.FlowEngine.Definitions;
using Hotline.Share.Enums.FlowEngine;
using SqlSugar;

namespace Hotline.FlowEngine.Workflows;

/// <summary>
/// 流转记录
/// </summary>
public class WorkflowTrace : StepBasicEntity
{

    public string StepId { get; set; }

    /// <summary>
    /// 流转记录状态
    /// </summary>
    public EWorkflowTraceStatus Status { get; set; }

    /// <summary>
    /// 过期时间（生成流转记录时取值当前workflow的过期时间）
    /// </summary>
    public DateTime ExpiredTime { get; set; }

    /// <summary>
    /// 办理时间限制(如：24小时、7个工作日)
    /// </summary>
    public string TimeLimit { get; set; }

    /// <summary>
    /// 会签从属关系
    /// </summary>
    [SugarColumn(IsNullable = true)]
    public string? ParentId { get; set; }

    /// <summary>
    /// 会签流转记录
    /// </summary>
    [SugarColumn(IsIgnore = true)]
    public List<WorkflowTrace> Traces { get; set; }

    #region method

    //public void Previous(string userId, string userName, string orgCode, string orgName) =>
    //    Complete(userId, userName, orgCode, orgName, EWorkflowTraceStatus.Back);

    //public void Recall(string userId, string userName, string orgCode, string orgName) =>
    //    Complete(userId, userName, orgCode, orgName, EWorkflowTraceStatus.Recall);

    public void Jump(string userId, string userName, string orgCode, string orgName, string opinion)
    {
        if (string.IsNullOrEmpty(AcceptUserId))
        {
            //未接办
            AcceptUserId = userId;
            AcceptUserName = userName;
            AcceptOrgCode = orgCode;
            AcceptOrgName = orgName;
            AcceptTime = DateTime.Now;
        }
        Complete(userId, userName, orgCode, orgName);
        Opinion = opinion;
    }

    public void Complete(string userId, string userName, string orgCode, string orgName)
    {
        OrgCode = orgCode;
        OrgName = orgName;
        UserId = userId;
        UserName = userName;
        CompleteTime = DateTime.Now;
    }

    #endregion
}