﻿
using SqlSugar;
using System.ComponentModel;
using System.Reflection.Metadata;
using XF.Domain.Entities;
using XF.Domain.Exceptions;
using XF.Domain.Repository;
using XF.Utility.EnumExtensions;

namespace Hotline.Settings;

[SugarIndex("unique_org_code", nameof(SystemOrganize.OrgCode), OrderByType.Desc, true)]
[Description("组织架构")]
public class SystemOrganize : CreationEntity
{
    /// <summary>
    /// 组织架构名称
    /// </summary>
    public string OrgName { get; set; }

    /// <summary>
    /// 组织架构简称
    /// </summary>
    [SugarColumn(IsNullable = true)]
    public string OrgShortName { get; set; }

    /// <summary>
    /// 组织架构Code（001001001） 解析：001,001,001）
    /// </summary>
    public string OrgCode { get; set; }

    /// <summary>
    /// 部门级别
    /// </summary>
    public int OrgLevel { get; set; }

    /// <summary>
    /// 部门类型
    /// </summary>
    public EOrgType OrgType { get; set; }

    [SugarColumn(IsIgnore = true)]
    public string OrgTypeText => OrgType.GetDescription();

    /// <summary>
    /// 上级ID
    /// </summary>
    [SugarColumn(IsNullable = true)]
    public string? ParentId { get; set; }

    /// <summary>
    /// 上级名称
    /// </summary>
    [SugarColumn(IsNullable = true)]
    public string? ParentName { get; set; }

    /// <summary>
    /// 是否启用
    /// </summary>
    public bool IsEnable { get; set; }

    [SugarColumn(IsIgnore = true)]
    public List<SystemOrganize> Children { get; set; }


    public void InitOrgLevel()
    {
        if (OrgCode.Length % 3 != 0)
            throw new UserFriendlyException("非法部门Code");
        OrgLevel = (OrgCode.Length / 3) - 1;
    }
}